@@ -175,14 +175,6 @@ void pk_backend_start_job(PkBackend *backend, PkBackendJob *job)
     pk_backend_job_set_user_data(job, apt);
 }
 
-/**
- * pk_backend_reset_job:
- */
-void pk_backend_reset_job(PkBackend *backend, PkBackendJob *job)
-{
-    // To keep the cache alive we just need to be defined...
-}
-
 /**
  * pk_backend_stop_job:
  */
