@@ -398,6 +398,7 @@ typedef enum {
 	PK_INFO_ENUM_DECOMPRESSING,
 	PK_INFO_ENUM_UNTRUSTED,
 	PK_INFO_ENUM_TRUSTED,
+	PK_INFO_ENUM_UNAVAILABLE,
 	PK_INFO_ENUM_LAST
 } PkInfoEnum;
 
@@ -473,6 +474,9 @@ typedef enum {
 	PK_TRANSACTION_FLAG_ENUM_ONLY_TRUSTED,		/* Since: 0.8.1 */
 	PK_TRANSACTION_FLAG_ENUM_SIMULATE,		/* Since: 0.8.1 */
 	PK_TRANSACTION_FLAG_ENUM_ONLY_DOWNLOAD,		/* Since: 0.8.1 */
+	PK_TRANSACTION_FLAG_ENUM_ALLOW_REINSTALL,	/* Since: 1.0.2 */
+	PK_TRANSACTION_FLAG_ENUM_JUST_REINSTALL,	/* Since: 1.0.2 */
+	PK_TRANSACTION_FLAG_ENUM_ALLOW_DOWNGRADE,	/* Since: 1.0.2 */
 	PK_TRANSACTION_FLAG_ENUM_LAST			/* Since: 0.8.1 */
 } PkTransactionFlagEnum;
 
