@@ -204,30 +204,33 @@ pk_backend_spawn_parse_stdout (PkBackendSpawn *backend_spawn,
 		}
 		pk_backend_job_package (job, info, sections[2], sections[3]);
 	} else if (g_strcmp0 (command, "details") == 0) {
-		if (size != 7 && size != 8) {
-			g_set_error (error, 1, 0, "invalid command'%s', size %i", command, size);
+		if (size != 8) {
+			g_set_error (error, 1, 0,
+				     "invalid command'%s', size %i",
+				     command, size);
 			return FALSE;
 		}
-		group = pk_group_enum_from_string (sections[3]);
+		group = pk_group_enum_from_string (sections[4]);
 
 		/* ITS4: ignore, checked for overflow */
-		package_size = atol (sections[6]);
+		package_size = atol (sections[7]);
 		if (package_size > 1073741824) {
-			g_set_error_literal (error, 1, 0, "package size cannot be larger than one Gb");
+			g_set_error_literal (error, 1, 0,
+					     "package size cannot be that large");
 			return FALSE;
 		}
-		g_strdelimit (sections[4], PK_UNSAFE_DELIMITERS, ' ');
+		g_strdelimit (sections[5], PK_UNSAFE_DELIMITERS, ' ');
 		if (!g_utf8_validate (sections[4], -1, NULL)) {
 			g_set_error (error, 1, 0,
 				     "text '%s' was not valid UTF8!",
-				     sections[4]);
+				     sections[5]);
 			return FALSE;
 		}
-		text = g_strdup (sections[4]);
+		text = g_strdup (sections[5]);
 		/* convert ; to \n as we can't emit them on stdout */
 		g_strdelimit (text, ";", '\n');
-		pk_backend_job_details (job, sections[1], size == 8 ? sections[7] : NULL, sections[2],
-					group, text, sections[5], package_size);
+		pk_backend_job_details (job, sections[1], sections[2], sections[3],
+					group, text, sections[6], package_size);
 		g_free (text);
 	} else if (g_strcmp0 (command, "finished") == 0) {
 		if (size != 1) {
@@ -779,6 +782,12 @@ pk_backend_spawn_get_envp (PkBackendSpawn *backend_spawn)
 	ret = pk_backend_job_get_interactive (priv->job);
 	g_hash_table_replace (env_table, g_strdup ("INTERACTIVE"), g_strdup (ret ? "TRUE" : "FALSE"));
 
+	/* UID */
+	ret = pk_backend_job_get_interactive (priv->job);
+	g_hash_table_replace (env_table,
+			      g_strdup ("UID"),
+			      g_strdup_printf ("%u", pk_backend_job_get_uid (priv->job)));
+
 	/* CACHE_AGE */
 	cache_age = pk_backend_job_get_cache_age (priv->job);
 	if (cache_age == G_MAXUINT) {
