@@ -105,6 +105,7 @@ typedef struct {
 	guint			 commit_id;
 	gulong			 finished_id;
 	gulong			 state_changed_id;
+	gulong			 allow_cancel_changed_id;
 	guint			 uid;
 	guint			 tries;
 } PkSchedulerItem;
@@ -152,6 +153,7 @@ PkTransaction *
 pk_scheduler_get_transaction (PkScheduler *scheduler, const gchar *tid)
 {
 	PkSchedulerItem *item;
+	g_return_val_if_fail (pk_is_thread_default (), NULL);
 	item = pk_scheduler_get_from_tid (scheduler, tid);
 	if (item == NULL)
 		return NULL;
@@ -173,6 +175,7 @@ pk_scheduler_role_present (PkScheduler *scheduler, PkRoleEnum role)
 	PkSchedulerItem *item;
 
 	g_return_val_if_fail (PK_IS_SCHEDULER (scheduler), FALSE);
+	g_return_val_if_fail (pk_is_thread_default (), FALSE);
 
 	/* check for existing transaction doing an update */
 	array = scheduler->priv->array;
@@ -202,6 +205,8 @@ pk_scheduler_item_free (PkSchedulerItem *item)
 		g_signal_handler_disconnect (item->transaction, item->finished_id);
 	if (item->state_changed_id != 0)
 		g_signal_handler_disconnect (item->transaction, item->state_changed_id);
+	if (item->allow_cancel_changed_id != 0)
+		g_signal_handler_disconnect (item->transaction, item->allow_cancel_changed_id);
 	g_object_unref (item->transaction);
 	if (item->commit_id != 0)
 		g_source_remove (item->commit_id);
@@ -246,6 +251,7 @@ pk_scheduler_remove (PkScheduler *scheduler, const gchar *tid)
 
 	g_return_val_if_fail (PK_IS_SCHEDULER (scheduler), FALSE);
 	g_return_val_if_fail (tid != NULL, FALSE);
+	g_return_val_if_fail (pk_is_thread_default (), FALSE);
 
 	item = pk_scheduler_get_from_tid (scheduler, tid);
 	if (item == NULL) {
@@ -513,6 +519,18 @@ pk_scheduler_commit (PkScheduler *scheduler, const gchar *tid)
 		pk_scheduler_run_item (scheduler, item);
 }
 
+/**
+ * pk_scheduler_transaction_allow_cancel_changed_cb:
+ **/
+static void
+pk_scheduler_transaction_allow_cancel_changed_cb (PkTransaction *transaction,
+					          gboolean allow_cancel,
+					          PkScheduler *scheduler)
+{
+	/* just proxy this back up */
+	g_signal_emit (scheduler, signals [PK_SCHEDULER_CHANGED], 0);
+}
+
 /**
  * pk_scheduler_transaction_state_changed_cb:
  **/
@@ -656,6 +674,7 @@ pk_scheduler_create (PkScheduler *scheduler,
 
 	g_return_val_if_fail (PK_IS_SCHEDULER (scheduler), FALSE);
 	g_return_val_if_fail (tid != NULL, FALSE);
+	g_return_val_if_fail (pk_is_thread_default (), FALSE);
 
 	/* already added? */
 	item = pk_scheduler_get_from_tid (scheduler, tid);
@@ -678,6 +697,10 @@ pk_scheduler_create (PkScheduler *scheduler,
 		g_signal_connect_after (item->transaction, "state-changed",
 					G_CALLBACK (pk_scheduler_transaction_state_changed_cb),
 					scheduler);
+	item->allow_cancel_changed_id =
+		g_signal_connect_after (item->transaction, "allow-cancel-changed",
+					G_CALLBACK (pk_scheduler_transaction_allow_cancel_changed_cb),
+					scheduler);
 
 	/* set transaction state */
 	pk_transaction_set_state (item->transaction, PK_TRANSACTION_STATE_NEW);
@@ -736,7 +759,7 @@ pk_scheduler_create (PkScheduler *scheduler,
  * pk_scheduler_get_locked:
  *
  * Return value: %TRUE if any of the transactions in progress are
- * locking a database or resource and cannot be cancelled.
+ * locking the packaging system.
  **/
 gboolean
 pk_scheduler_get_locked (PkScheduler *scheduler)
@@ -747,16 +770,12 @@ pk_scheduler_get_locked (PkScheduler *scheduler)
 	_cleanup_ptrarray_unref_ GPtrArray *array = NULL;
 
 	g_return_val_if_fail (PK_IS_SCHEDULER (scheduler), FALSE);
-
-	/* anything running? */
-	array = pk_scheduler_get_active_transactions (scheduler);
-	if (array->len == 0)
-		return FALSE;
+	g_return_val_if_fail (pk_is_thread_default (), FALSE);
 
 	/* check if any backend in running transaction is locked at time */
+	array = pk_scheduler_get_active_transactions (scheduler);
 	for (i = 0; i < array->len; i++) {
-		item = (PkSchedulerItem *) g_ptr_array_index (array, i);
-
+		item = g_ptr_array_index (array, i);
 		job = pk_transaction_get_backend_job (item->transaction);
 		if (job == NULL)
 			continue;
@@ -766,6 +785,36 @@ pk_scheduler_get_locked (PkScheduler *scheduler)
 	return FALSE;
 }
 
+/**
+ * pk_scheduler_get_inhibited:
+ *
+ * Return value: %TRUE if any of the transactions in progress cannot be
+ * cancelled.
+ **/
+gboolean
+pk_scheduler_get_inhibited (PkScheduler *scheduler)
+{
+	PkBackendJob *job;
+	PkSchedulerItem *item;
+	guint i;
+	_cleanup_ptrarray_unref_ GPtrArray *array = NULL;
+
+	g_return_val_if_fail (PK_IS_SCHEDULER (scheduler), FALSE);
+	g_return_val_if_fail (pk_is_thread_default (), FALSE);
+
+	/* check if any backend in running transaction is locked at time */
+	array = pk_scheduler_get_active_transactions (scheduler);
+	for (i = 0; i < array->len; i++) {
+		item = g_ptr_array_index (array, i);
+		job = pk_transaction_get_backend_job (item->transaction);
+		if (job == NULL)
+			continue;
+		if (!pk_backend_job_get_allow_cancel (job))
+			return TRUE;
+	}
+	return FALSE;
+}
+
 /**
  * pk_scheduler_cancel_background:
  **/
@@ -778,6 +827,7 @@ pk_scheduler_cancel_background (PkScheduler *scheduler)
 	PkTransactionState state;
 
 	g_return_if_fail (PK_IS_SCHEDULER (scheduler));
+	g_return_if_fail (pk_is_thread_default ());
 
 	/* cancel all running background transactions */
 	array = scheduler->priv->array;
@@ -806,6 +856,7 @@ pk_scheduler_cancel_queued (PkScheduler *scheduler)
 	PkTransactionState state;
 
 	g_return_if_fail (PK_IS_SCHEDULER (scheduler));
+	g_return_if_fail (pk_is_thread_default ());
 
 	/* clear any pending transactions */
 	array = scheduler->priv->array;
@@ -832,6 +883,7 @@ pk_scheduler_get_array (PkScheduler *scheduler)
 	_cleanup_ptrarray_unref_ GPtrArray *parray = NULL;
 
 	g_return_val_if_fail (PK_IS_SCHEDULER (scheduler), NULL);
+	g_return_val_if_fail (pk_is_thread_default (), NULL);
 
 	/* use a temp array, as not all are in progress */
 	parray = g_ptr_array_new_with_free_func (g_free);
@@ -858,6 +910,7 @@ guint
 pk_scheduler_get_size (PkScheduler *scheduler)
 {
 	g_return_val_if_fail (PK_IS_SCHEDULER (scheduler), 0);
+	g_return_val_if_fail (pk_is_thread_default (), 0);
 	return scheduler->priv->array->len;
 }
 
